Method: Rubyvis::Mark.defaults

Defined in:
lib/rubyvis/mark.rb

.defaultsObject

Default properties for all mark types. By default, the data array is the parent data as a single-element array; if the data property is not specified, this causes each mark to be instantiated as a singleton with the parents datum. The visible property is true by default, and the reverse property is false.



480
481
482
# File 'lib/rubyvis/mark.rb', line 480

def self.defaults
  Mark.new({:data=>lambda {|d| [d]}, :visible=>true, :antialias=>true, :events=>'painted'})
end