Method: Rubyvis::Layout::Network#reset
- Defined in:
- lib/rubyvis/layout/network.rb
#reset ⇒ Object
Resets the cache, such that changes to layout property definitions will be visible on subsequent render. Unlike normal marks (and normal layouts), properties associated with network layouts are not automatically re-evaluated on render; the properties are cached, and any expensive layout algorithms are only run after the layout is explicitly reset.
(@returns Rubyvis::Layout::Network) self
204 205 206 207 |
# File 'lib/rubyvis/layout/network.rb', line 204 def reset self._id=Rubyvis.id() self end |